#!/bin/bash

# This script is edited based on tyfamily_integrated_all_slurm_merge.sh
# downsample to 50x if greater.

# parsing arguments
POSITIONAL_ARGS=()
# mcc_dir="${mcc_dir}",out_dir="${out_dir}",ref_fa="${ref_fa}",te_seqs="${te_seqs}",cov_te_seqs="${cov_te_seqs}",data_dir="${data_dir}",sample_name="${sample_name}",tax="${tax}",location="${location}"
while [[ $# -gt 0 ]]; do
  case $1 in
    -m|--mcc_dir)
      mcc_dir="$2"
      shift # past argument
      shift # past value
      ;;
    -o|--out_dir)
      out_dir="$2"
      shift # past argument
      shift # past value
      ;;
    -r|--ref_fa)
      ref_fa="$2"
      shift # past argument
      shift # past value
      ;;
    -e|--te_seqs)
      te_seqs="$2"
      shift # past argument
      shift # past value
      ;;
    -c|--cov_te_seqs)
      cov_te_seqs="$2"
      shift # past argument
      shift # past value
      ;;
    -d|--data_dir)
      data_dir="$2"
      shift # past argument
      shift # past value
      ;;
    -s|--sample_name)
      sample_name="$2"
      shift # past argument
      shift # past value
      ;;
    -t|--tax)
      tax="$2"
      shift # past argument
      shift # past value
      ;;
    -l|--location)
      location="$2"
      shift # past argument
      shift # past value
      ;;
    -p|--threads)
      threads="$2"
      shift # past argument
      shift # past value
      ;;    
    -*|--*)
      echo "Unknown option $1"
      exit 1
      ;;
    *)
      POSITIONAL_ARGS+=("$1") # save positional arg
      shift # past argument
      ;;
  esac
done

set -- "${POSITIONAL_ARGS[@]}" # restore positional parameters

echo "mcc_dir = ${mcc_dir}"
echo "out_dir = ${out_dir}"
echo "ref_fa = ${ref_fa}"
echo "te_seqs = ${te_seqs}"
echo "cov_te_seqs = ${cov_te_seqs}"
echo "data_dir = ${data_dir}"
echo "sample_name = ${sample_name}"
echo "tax = ${tax}"
echo "location = ${location}"
echo "threads = ${threads}"

cd $data_dir

CONDA_BASE=$(conda info --base)
source ${CONDA_BASE}/etc/profile.d/conda.sh
conda activate mcc_sim

# check if merged
sample_count=`echo $sample_name | sed "s/_/\n/g" | wc -l` 
if [[ $sample_count -eq 1 ]]; then 
    echo "[INFO] ${sample_name} is a single run"
    if [[ ! -e $data_dir/${sample_name}.sra ]]; then 
        prefetch $sample_name -o $data_dir/${sample_name}.sra
    fi
    vdb-validate $data_dir/${sample_name}.sra &> md5_out_${sample_name}
    stat=`grep "'${sample_name}.sra' is consistent" md5_out_${sample_name}`
    count=0
    while [[ -z $stat ]] && [[ $count -lt 10 ]]; do # check if the string is empty
        prefetch $sample_name -o $data_dir/${sample_name}.sra
        vdb-validate $data_dir/${sample_name}.sra &> md5_out_${sample_name}
        stat=`grep "'${sample_name}.sra' is consistent" md5_out_${sample_name}`
        (( count++ ))
    done
    if [[ -e $data_dir/${sample_name}.sra ]]; then
        fastq-dump -O $data_dir $data_dir/${sample_name}.sra --split-e
        echo "Download success for ${sample_name}!"
    else
        echo "Download failed for ${sample_name}!"; exit 1;
    fi
    rm md5_out_${sample_name}
elif [[ $sample_count -gt 1 ]]; then 
    echo "[INFO] ${sample_name} is a merged run"
    # generate run accession list
    echo $sample_name | sed "s/_/\n/g" > $data_dir/${sample_name}.list
    # create empty merged files
    > $data_dir/${sample_name}_1.fastq
    > $data_dir/${sample_name}_2.fastq
    > $data_dir/${sample_name}.fastq
    while read SRR; do
        if [[ ! -e $data_dir/${SRR}.sra ]]; then 
            prefetch $SRR -o $data_dir/${SRR}.sra
        fi
        vdb-validate $data_dir/${SRR}.sra &> md5_out_${SRR}
        stat=`grep "'${SRR}.sra' is consistent" md5_out_${SRR}`
        count=0
        while [[ -z $stat ]] && [[ $count -lt 10 ]]; do # check if the string is empty
            prefetch $SRR -o $data_dir/${SRR}.sra
            vdb-validate $data_dir/${SRR}.sra &> md5_out_${SRR}
            stat=`grep "'${SRR}.sra' is consistent" md5_out_${SRR}`
            (( count++ ))
        done
        if [[ -e $data_dir/${SRR}.sra ]]; then
            fastq-dump -O $data_dir $data_dir/${SRR}.sra --split-e
            echo "Download success for ${SRR}!"
        else
            echo "Download failed for ${SRR}!"; exit 1;
        fi
        cat $data_dir/${SRR}_1.fastq >> $data_dir/${sample_name}_1.fastq
        cat $data_dir/${SRR}_2.fastq >> $data_dir/${sample_name}_2.fastq
        cat $data_dir/${SRR}.fastq >> $data_dir/${sample_name}.fastq
        rm md5_out_${SRR}
    done < $data_dir/${sample_name}.list
fi

# downsample to 50x if vailable
fq1=`ls $data_dir/${sample_name}_1.fastq`
fq2=`ls $data_dir/${sample_name}_2.fastq`
fq=`ls $data_dir/${sample_name}.fastq`
sed -i 's/length=.*//g' $fq1
sed -i 's/length=.*//g' $fq2
sed -i 's/length=.*//g' $fq
if [[ (-z "${fq}") || (! -s ${fq}) ]]; then # if fq is empty
    echo "[INFO] Data in paired end mode"
    seqkit stats $data_dir/${sample_name}_1.fastq > $data_dir/${sample_name}_1.stats
    sum_len=`sed '1d' $data_dir/${sample_name}_1.stats | awk '{print $5}' | sed 's/,//g'`
    # estimate fold coverage
    est_cov=$((sum_len * 2 / 12000000))
    avg_len=`sed '1d' $data_dir/${sample_name}_1.stats | awk '{print $7}' | sed 's/\..*$//g'`
    # number of reads needed for 50x
    num_read=$((12000000 * 50 / 2 / avg_len))
    if [[ $est_cov -gt 50 ]]; then
        seqtk sample -s233 $data_dir/${sample_name}_1.fastq ${num_read} > $data_dir/${sample_name}_sub_1.fastq
        seqtk sample -s233 $data_dir/${sample_name}_2.fastq ${num_read} > $data_dir/${sample_name}_sub_2.fastq
        # rename fastq
        mv $data_dir/${sample_name}_1.fastq $data_dir/${sample_name}_all_1.fastq
        mv $data_dir/${sample_name}_2.fastq $data_dir/${sample_name}_all_2.fastq
        mv $data_dir/${sample_name}_sub_1.fastq $data_dir/${sample_name}_1.fastq
        mv $data_dir/${sample_name}_sub_2.fastq $data_dir/${sample_name}_2.fastq
    fi
else
    echo "[INFO] Data in single end mode"
    seqkit stats $data_dir/${sample_name}.fastq > $data_dir/${sample_name}.stats
    sum_len=`sed '1d' $data_dir/${sample_name}.stats | awk '{print $5}' | sed 's/,//g'`
    # estimate fold coverage
    est_cov=$((sum_len / 12000000))
    avg_len=`sed '1d' $data_dir/${sample_name}.stats | awk '{print $7}' | sed 's/\..*$//g'`
    # number of reads needed for 50x
    num_read=$((12000000 * 50 / avg_len))
    if [[ $est_cov -gt 50 ]]; then
        seqtk sample -s233 $data_dir/${sample_name}.fastq ${num_read} > $data_dir/${sample_name}_sub.fastq
        # rename fastq
        mv $data_dir/${sample_name}.fastq $data_dir/${sample_name}_all.fastq
        mv $data_dir/${sample_name}_sub.fastq $data_dir/${sample_name}.fastq
    fi
fi
conda deactivate

# run mcclintock
conda activate mcclintock
rm -rf $out_dir/${sample_name}
mkdir -p $out_dir/${sample_name}
fq1=`ls $data_dir/${sample_name}_1.fastq`
fq2=`ls $data_dir/${sample_name}_2.fastq`
fq=`ls $data_dir/${sample_name}.fastq`
# sed -i 's/length=.*//g' $fq1
# sed -i 's/length=.*//g' $fq2
# sed -i 's/length=.*//g' $fq
if [[ (-z "${fq}") || (! -s ${fq}) ]]; then # if fq is empty
    echo "[INFO] Submit McC in paired end mode"
    python3 $mcc_dir/mcclintock.py -o $out_dir/${sample_name} -r $ref_fa -c $te_seqs -s $cov_te_seqs -g $location -t $tax -1 $fq1 -2 $fq2 -p ${threads} --keep_intermediate minimal --vcf siteonly,sample
else
    echo "[INFO] Submit McC in single end mode"
    python3 $mcc_dir/mcclintock.py -o $out_dir/${sample_name} -r $ref_fa -c $te_seqs -s $cov_te_seqs -g $location -t $tax -1 $fq -p ${threads} --keep_intermediate minimal --vcf siteonly,sample
fi

if [[ (-e $out_dir/${sample_name}/${sample_name}_1/results/summary/te_summary.csv) || (-e $out_dir/${sample_name}/${sample_name}/results/summary/te_summary.csv) ]]; then
  cp $out_dir/${sample_name}/${sample_name}*/results/summary/te_summary.csv $out_dir/${sample_name}
  sample_count=`echo $sample_name | sed "s/_/\n/g" | wc -l` 
  if [[ $sample_count -gt 1 ]]; then
    while read SRR; do
        rm $data_dir/${SRR}*
    done < $data_dir/${sample_name}.list
    rm $data_dir/${sample_name}.list
  fi
  rm $data_dir/${sample_name}*fastq
  rm $data_dir/${sample_name}.sra
  echo "[INFO] McC finished successfully!"
else
  echo "[INFO] McC failed for ${sample_name}!"
fi
conda deactivate

