# modified based on /home/jc33471/jingxuan/src/rscripts/mcc_paper_realdata_sapelo2.R
require(tools)
library(ggplot2)
library(plyr); library(dplyr)
library(tidyr)
library(Rmisc)
library(ggpubr)
library(reshape2)
library(GGally)
pdf(NULL)

# setup working directory
args = commandArgs(trailingOnly=TRUE)
if (length(args) != 3) {
  stop("Usage: Rscript --vanilla empirical_r_vis_data.R <mcc_dir> <emp_dir> <prefix>", call.=FALSE)
} else if (length(args) == 3) {
  mcc_dir <- args[1]
  emp_dir <- args[2]
  pub <- args[3]
}

results.location <- paste0(emp_dir,"/out/")
output.location <- paste0(emp_dir,"/r_vis/")
dir.create(output.location, showWarnings = FALSE, recursive = TRUE)

##################### TSD length ##################### 
simulation_tsd_len <- function(te_list){
    # settings for methods/title
    srmethods <- c("ngs_te_mapper","ngs_te_mapper2","relocate","relocate2","tebreak","temp")
    title2 <- c("ngs_te_mapper","RelocaTE","TEMP","RetroSeq","PoPoolationTE","TE-locate","TE-locate","ngs_te_mapper2","RelocaTE2","TEMP2","TEFLoN","PoPoolationTE2","TEBreak","Coverage")
    names(title2) <- c("ngs_te_mapper","relocate","temp","retroseq","popoolationte","te-locate","telocate","ngs_te_mapper2","relocate2","temp2","teflon","popoolationte2","tebreak","coverage")

    # path to tsd data
    tsd_dir <- paste0(output.location, "/tsd_data/")
    dir.create(tsd_dir, showWarnings = FALSE, recursive = TRUE)

    
    all.bed <- vector()
    for (method in srmethods){
        bedfile <- list.files(Sys.glob(paste0(results.location, "/*/*_1/results/", method)), full.names=T, pattern=paste0(method, "_nonredundant.bed$"))
        all.bed <- c(all.bed, bedfile)
    }
    
    # For each bed file
    system(paste0("> ", tsd_dir, "/all.bed"))
    for(i in 1:length(all.bed))
    {
        system(paste("grep non- ", all.bed[i], " >> ", tsd_dir, "/all.bed" ,sep=""))
    }
    dataset <- read.table(paste(tsd_dir, "/all.bed" ,sep=""), header=FALSE,col.names=c("Chromosome","Start","End", "TE Name", "Score", "Orientation"),sep="\t")
    
    dataset[grep("ngs_te_mapper",  dataset$TE.Name), "Method"] <- "ngs_te_mapper"
    dataset[grep("ngs_te_mapper2",  dataset$TE.Name), "Method"] <- "ngs_te_mapper2"
    dataset[grep("relocate",  dataset$TE.Name), "Method"] <- "relocate"
    dataset[grep("relocate2",  dataset$TE.Name), "Method"] <- "relocate2"
    dataset[grep("temp_sr",  dataset$TE.Name), "Method"] <- "temp"
    dataset[grep("temp_rp",  dataset$TE.Name), "Method"] <- "tempreadpair"
    dataset[grep("temp_nonab",  dataset$TE.Name), "Method"] <- "tempreference"
    dataset[grep("temp2_rp",  dataset$TE.Name), "Method"] <- "temp2readpair"
    dataset[grep("temp2_nonab",  dataset$TE.Name), "Method"] <- "temp2reference"
    dataset[grep("temp\\|sr",  dataset$TE.Name), "Method"] <- "temp"
    dataset[grep("temp\\|rp",  dataset$TE.Name), "Method"] <- "tempreadpair"
    dataset[grep("temp\\|nonab",  dataset$TE.Name), "Method"] <- "tempreference"
    dataset[grep("temp2\\|rp",  dataset$TE.Name), "Method"] <- "temp2readpair"
    dataset[grep("temp2\\|nonab",  dataset$TE.Name), "Method"] <- "temp2reference"
    dataset[grep("retroseq",  dataset$TE.Name), "Method"] <- "retroseq"
    dataset[grep("telocate",  dataset$TE.Name), "Method"] <- "telocate"
    dataset[grep("te-locate",  dataset$TE.Name), "Method"] <- "telocate"
    dataset[grep("popoolationte",  dataset$TE.Name), "Method"] <- "popoolationte"
    dataset[grep("popoolationte2",  dataset$TE.Name), "Method"] <- "popoolationte2"
    dataset[grep("teflon",  dataset$TE.Name), "Method"] <- "teflon"
    dataset[grep("tebreak",  dataset$TE.Name), "Method"] <- "tebreak"
    
    for (te in te_list){
        dataset[grep(paste0(te,"\\|"),  dataset$TE.Name), "TE"] <- te
    }

    dataset$tsd <- dataset$End - dataset$Start
    # new.TEs <- grep("non-reference",dataset$TE.Name)
    
    # Count the number of TEs per sample for each method
    # newTEs <- paste(dataset$Method[new.TEs],dataset$Sample[new.TEs])
    # new.TE.counts <- table(newTEs)
    get_ty_plot <- function(method,te){
        dataset_plot <- dataset %>% filter(Method==method & TE==te)
        p1 <- dataset_plot %>% ggplot(aes(x = tsd)) +
            xlim(c(0,15)) +
            geom_histogram(binwidth = 1, color="brown", fill="white") +
            theme_bw() +
            theme(axis.text.x = element_text(), axis.ticks.x = element_line(),
                    axis.text.y = element_text(angle = 90, hjust = 0.5), axis.ticks.y = element_line(),
                    plot.title = element_text(face = "bold", hjust = 0.5, size = 14)) +
            scale_y_continuous(breaks = function(x) seq.int(0, floor(x[2]), by = ceiling((floor(x[2])-0)/4))) +
            { if(length(dataset_plot$tsd) > 0) geom_text(x = 13, aes(y = max(table(tsd)), label = paste0("5bp=",scales::percent(length(tsd[tsd==5])/length(tsd)))), colour="brown") } +
            labs(x = NULL, y = NULL, title = title2[method])
        return(p1)
    }

    for (TE in te_list){
        plotSets <- lapply(srmethods,get_ty_plot,te=TE)
        out_p <- ggpubr::ggarrange(plotlist = plotSets, nrow = 1)
        out_p2 <- annotate_figure(out_p, left = text_grob(paste0(TE, " count"), rot = 90, size = 10) , bottom = text_grob("Predicted TSD lengths (bp)", size = 10))
        ggsave(plot = out_p2,
            paste0(output.location, "/realtsd_",TE,".pdf"), device = "pdf", width = 20, height = 4, units = "in", dpi = 320)
    }
  
}
TEs <- c("TY1", "TY2", "TY3", "TY4")
simulation_tsd_len(TEs)
  

##################### create table for components for strain ##################### 

# meta <- read.csv("~/jingxuan/data/cerevisiae/integration/all_scer_meta_merged.csv", stringsAsFactors = F)
# meta_dataset <- meta %>% filter(dataset=="strope"|dataset=="peter")
run_accession <- as.vector(system(paste0("ls ", results.location, " | grep -v log"), intern = T))
results_path <- results.location
df_box <- data.frame()
methods <- c("ngs_te_mapper","ngs_te_mapper2","relocate","relocate2","tebreak","temp","temp2","retroseq","popoolationte","popoolationte2","te-locate","teflon")
for(strain in run_accession)
{
  # split read methods
  for(method in methods)
  {
    count <- system(paste("cat ", results_path, "/", strain, "/*/results/", method, "/*_nonredundant.bed | grep non-reference | wc | awk '{print $1}'",sep=""), intern=TRUE)
    strain_line <- cbind(Strain = strain, Count = count, Method = method)
    df_box <- rbind(df_box, as.data.frame(strain_line))
  }

  
}

# export csv file for main figure visualization
write.csv(df_box, paste0(output.location, "/method_table_for_strain_",pub,".csv"))

##################### coverage results ###########################

# meta <- read.csv("~/jingxuan/data/cerevisiae/integration/all_scer_meta_merged.csv", stringsAsFactors = F)
# meta_dataset <- meta %>% filter(dataset=="strope"|dataset=="peter")
run_accession <- as.vector(system(paste0("ls ", results.location, " | grep -v log"), intern = T))
df_all <- data.frame()
for(strain in run_accession)
{
  tmp_cov <- read.csv(paste0(results.location, strain, "/", strain, "_1/results/coverage/te_depth.csv"), stringsAsFactors = F, row.names = 1) %>%
    select(-Normalized.Unique.Depth) %>%
    t %>% as.data.frame %>% 
    select(contains("_GAG") | contains("_I"))
    # mutate(TY1_I = sum(TY1_GAG, TY1prime_GAG, TY1novel_GAG)) %>%
    # select(-contains("_GAG"))
  rownames(tmp_cov) <- strain
  df_all <- rbind(df_all, tmp_cov)
}
df_sum <- data.frame(TE = c("TY1", "TY2", "TY3", "TY3_1p", "TY4", "TSU4", "TY5"), total_cov = c(sum(df_all$TY1_I), sum(df_all$TY2_I), sum(df_all$TY3_I), sum(df_all$TY3_1p_I), sum(df_all$TY4_I), sum(df_all$TSU4_I), sum(df_all$TY5_I)))
write.csv(df_sum, paste0(output.location, "/coverage_table_",pub,".csv"))
write.csv(df_all, paste0(output.location, "/coverage_table_for_strain_",pub,".csv"))

df_sum$TE <- factor(df_sum$TE, levels = c("TY1","TY2","TY3","TY3_1p","TY4","TSU4","TY5"))

p <- df_sum %>% ggplot( aes(x=TE, y=total_cov)) +
  geom_bar(stat="identity", width = 0.6, fill = "#cc4c02") +
  labs(x = NULL, y = NULL, title = "Coverage") +
  theme_bw() +
  theme(
    axis.text.x = element_text(angle = 45, vjust = 1, hjust=1),
    plot.title = element_text(hjust = 0.5)) +
  scale_fill_manual(values = c("#cc4c02")) +
  scale_x_discrete(labels = c("TY1"="Ty1","TY2"="Ty2","TY3"="Ty3","TY3_1p"="Ty3_1p","TY4"="Ty4","TSU4"="Tsu4","TY5"="Ty5"))


ggsave(paste0(output.location, "/coverage_bar_",pub,".pdf"), plot = p,  device = "pdf", width = 3, height = 4, units = "in", dpi = 320)

##################### tRNA table ##################### 
system(paste("awk 'BEGIN{OFS=FS=\"\\t\"}{if($7==\"+\"){$5=$4;}else if($7==\"-\"){$4=$5;} print;}' ", mcc_dir,"/test/tRNA_coordinates.gff > ", output.location, "/sacCer2tRNAstarts.gff", sep=""))
system(paste0("sort -k1,1 -k4,4n ",output.location,"/sacCer2tRNAstarts.gff > ",output.location,"/sacCer2tRNAstarts_sorted.gff"))
#system(paste("awk '{print $1\"\\t\"$2\"\\t\"$3\"\\t\"$4\"\\t\"$4\"\\t\"$6\"\\t\"$7\"\\t\"$8\"\\t\"$9}' ", mcc_dir,"/test/tRNA_coordinates.gff > ", output.location, "/sacCer2tRNAstarts.gff", sep=""))
results_path <- results.location
# methods <- c("ngs_te_mapper","relocate","temp","retroseq","popoolationte","te-locate")
methods <- c("ngs_te_mapper","ngs_te_mapper2","relocate","relocate2","tebreak","temp","temp2","retroseq","popoolationte","popoolationte2","te-locate","teflon")
# TEs <- system(paste("grep \\> ", results_path, "/SRR800770/sacCer2/consensus_fasta/consensusTEs.fasta | sed 's/>//g' ",  sep=""), intern=TRUE)
TEs <- c("TY1", "TY2", "TY3", "TY3_1p", "TY4", "TSU4", "TY5")
tRNAtable <- vector()
#TE <- "TY1"
#method <- "ngs_te_mapper"
for(TE in TEs)
{
  TEline <- vector()
  for(method in methods)
  {
    bed_file <- paste(results_path, "/*/*/results/", method, "/*_nonredundant.bed", sep = "")
    system(paste("grep -h ", TE, "\\| ", bed_file, "| grep non-reference > ", output.location, "/", method, TE, ".list",sep=""))
    intRNA <- system(paste("bedtools window -u -l 1000 -r 500 -sw -a ", output.location, "/", method, TE, ".list -b ", output.location, "/sacCer2tRNAstarts.gff | wc  | awk '{print $1}'", sep=""), intern=TRUE)
    total <- system(paste("grep -h ", TE, "\\| ", bed_file, "| grep non-reference | wc  | awk '{print $1}'",  sep=""), intern=TRUE)
    nottRNA <- as.numeric(total) - as.numeric(intRNA)
    #percent<-sprintf("%.2f", (as.numeric(intRNA)/as.numeric(total))*100, 2)
    percent<-round((as.numeric(intRNA)/as.numeric(total))*100)
    if (is.nan(percent)) 
    {
      percent<-"N.A."
      temethodresult<-paste(intRNA, "/", total, " (", percent, ")",sep="")
    }
    else 
    {
      temethodresult<-paste(intRNA, "/", total, " (", percent, "%)",sep="")
    }
    TEline<-cbind(TEline, temethodresult)
    system(paste("rm ", output.location, "/", method, TE, ".list",sep=""))
  }
  tRNAtable<-rbind(tRNAtable, TEline)
}
rownames(tRNAtable) <- TEs
colnames(tRNAtable) <- methods
write.csv(tRNAtable, paste(output.location, "/tRNAtable_",pub,".csv", sep = ""))

##################### create dist to tRNA tables ##################### 

## dist to tRNA for all methods
trna_method <- function(method){
    all.bed <- vector()
    bedfile <- list.files(Sys.glob(paste0(results.location, "/*/*_1/results/", method)), full.names=T, pattern=paste0(sub("-","",method), "_nonredundant.bed$"))
    all.bed <- c(all.bed, bedfile)

    # path to dist data
    density_dir <- paste0(output.location, "/density_data/")
    dir.create(density_dir, showWarnings = FALSE, recursive = TRUE)
    
    system(paste0("rm -f ", density_dir, "/",method,"all.bed"))
    for(i in 1:length(all.bed))
    {
        system(paste("grep non- ", all.bed[i], " >> ", density_dir, "/",method,"all.bed" ,sep=""))
    }
    
    for(TE in c("TY1","TY2","TY3","TY4")){
        system(paste("grep --no-filename ", TE, "\\| ", density_dir, "/",method,"all.bed | grep non-reference | sort -k 1,1 > ", density_dir, "/", TE, "_",method,".bed", sep=""))
        # create file for main figure vis
        system(paste0("bedtools window -u -l 2000 -r 500 -sw -a ",density_dir,"/",TE,"_",method,".bed -b ",output.location,"/sacCer2tRNAstarts_sorted.gff | sort -k1,1 -k2,2n > ",density_dir,"/",TE,"_",method,"_window.bed"))
        bed_filter <- read.table(paste0(density_dir,"/",TE,"_",method,"_window.bed"), stringsAsFactors = F) %>%
          dplyr::mutate(V4 = gsub("\\|.*$","",V4,perl = T)) %>%
          dplyr::mutate(V3 = V2+1) %>%
          # dplyr::mutate(V3 = round(rowMeans(cbind(V2, V3)), digits = 0)) %>%
          # dplyr::mutate(V2 = V3-1) %>%
          distinct()
        write.table(bed_filter, paste0(density_dir,"/",TE,"_",method,"_filter.bed"), col.names = F, row.names = F, quote = F, sep = "\t")
        system(paste0("sort -k1,1 -k2,2n ", density_dir,"/",TE,"_",method,"_filter.bed > ", density_dir,"/",TE,"_",method,"_filter_sort.bed"))
        system(paste0("bedtools closest -D b -a ",density_dir,"/",TE,"_",method,"_filter_sort.bed -b ",output.location,"/sacCer2tRNAstarts_sorted.gff > ",density_dir,"/",TE,"_",method,"_dist.out"))
    }
}

methods <- c("ngs_te_mapper","ngs_te_mapper2","relocate","relocate2","tebreak","temp","temp2","retroseq","popoolationte","popoolationte2","te-locate","teflon")
for(i in methods){
  trna_method(i)
}
