# modified based on /home/jc33471/jingxuan/src/rscripts/mcc_paper_realdata_sapelo2.R
require(tools)
library(ggplot2)
library(plyr); library(dplyr)
library(tidyr)
library(Rmisc)
library(ggpubr)
library(reshape2)
library(GGally)
pdf(NULL)

# setup working directory
args = commandArgs(trailingOnly=TRUE)
if (length(args) != 3) {
  stop("Usage: Rscript --vanilla empirical_r_vis_plots.R <data_dir> <emp_dir> <prefix>", call.=FALSE)
} else if (length(args) == 3) {
  data_dir <- args[1]
  emp_dir <- args[2]
  pub <- args[3]
}

results.location <- paste0(emp_dir,"/out/")
output.location <- paste0(emp_dir,"/r_vis/")
dir.create(output.location, showWarnings = FALSE, recursive = TRUE)

############################ real data - box plot #############################
method_table <- read.csv(paste0(output.location, "/method_table_for_strain_",pub,".csv"), stringsAsFactors = F, row.names = 1)

all_table <- rbind(method_table) %>% 
  mutate(Method = gsub("te-locate","te.locate",Method)) %>%
  mutate(Dev = case_when(Method %in% c("ngs_te_mapper","relocate","temp","retroseq","popoolationte","te.locate") ~ "old",
                         TRUE ~ "new"))

# Get the median of best-in-class methods
best_tool <- filter(method_table, Method %in% c("temp","temp2","retroseq","relocate2","tebreak"))
best_median <- median(best_tool$Count)

# modified title
title <- c("ngs_te_mapper","RelocaTE","TEMP","RetroSeq","PoPoolationTE","TE-locate","ngs_te_mapper2","RelocaTE2","TEMP2","TEFLoN","PoPoolationTE2","TEBreak","Coverage")
names(title) <- c("ngs_te_mapper","relocate","temp","retroseq","popoolationte","te.locate","ngs_te_mapper2","relocate2","temp2","teflon","popoolationte2","tebreak","coverage")

setcolors <- c(
  "ngs_te_mapper" = "#E69F0070",
  "ngs_te_mapper2" = "#E69F00",
  "relocate" = "#56B4E970",
  "relocate2" = "#56B4E9",
  "temp" = "#AFD3C3",
  "temp2" = "#009E73",
  "popoolationte" = "#F0E44270",
  "popoolationte2" = "#F0E442",
  "retroseq" = "#0072B2",
  "te.locate" = "#CCCCCC",
  "teflon" = "#D55E00",
  "tebreak" = "#CC79A7",
  "coverage" = "#DDCC77"
)

all_table %>% filter(Method != "coverage") %>% ggplot(aes(x = Method, y = Count)) +
  theme_bw() +
  geom_hline(yintercept = best_median, lwd = .5, color = "gray40", linetype="dashed") +
  #geom_hline(yintercept = 20, lwd = .5, color = "gray60", linetype="dashed") +
  geom_boxplot(aes(fill = Method), width=0.5) +
  scale_y_continuous(trans='log10', limits = c(1,500),breaks = c(1,2,5,10,20,50,100,200,500)) +
  scale_fill_manual(values = setcolors) +
  labs(x = NULL) +
  theme(plot.title = element_text(hjust = 0.5, size = 14), text = element_text(size = 15),
        axis.text.x = element_blank(), axis.ticks.x = element_blank(),
        strip.background=element_rect(colour="white", fill="white"), strip.text = element_text(size = 15, angle = 0),
        legend.position = "none",
        panel.grid.major.x = element_blank(),
        panel.spacing=unit(1.9, "lines")) +
  facet_wrap(~factor(Method, levels = c("ngs_te_mapper","relocate","temp","popoolationte","retroseq","te.locate","ngs_te_mapper2","relocate2","temp2","popoolationte2","teflon","tebreak")),
             ncol = 6, nrow = 2, scales = "free_x", labeller = as_labeller(title),
             strip.position="top")
ggsave(paste0(output.location, "/nonref_",pub,"_boxplot.pdf"), device = "pdf", width = 14, height = 8, units = "in", dpi = 320)

############################ real data - strain total correlation ###########################
method_table <- read.csv(paste0(output.location, "/method_table_for_strain_",pub,".csv"), stringsAsFactors = F, row.names = 1) %>%
  mutate(Method = gsub("te-locate","te.locate", Method)) %>%
  spread(key = Method, value = Count) %>%
  select(-Strain)

title <- c("ngs_te_mapper","RelocaTE","TEMP","RetroSeq","PoPoolationTE","TE-locate","ngs_te_mapper2","RelocaTE2","TEMP2","TEFLoN","PoPoolationTE2","TEBreak","Coverage")
names(title) <- c("ngs_te_mapper","relocate","temp","retroseq","popoolationte","te.locate","ngs_te_mapper2","relocate2","temp2","teflon","popoolationte2","tebreak","coverage")

low_color <- "#2166ac"
high_color <- "#b2182b"
# function
upper_fn <- function(data, mapping, method="pearson", use="pairwise", ...){
  # grab data
  x <- eval_data_col(data, mapping$x)
  y <- eval_data_col(data, mapping$y)
  
  # calculate correlation
  corr <- cor(x, y, method=method, use=use)
  
  # calculate colour based on correlation value
  # Here I have set a correlation of minus one to blue, 
  # zero to white, and one to red 
  # Change this to suit: possibly extend to add as an argument of `my_fn`
  colFn <- colorRampPalette(c(low_color, "white", high_color), interpolate ='spline')
  fill <- colFn(100)[findInterval(corr, seq(0, 1, length=100))]
  
  ggally_cor(data = data, mapping = mapping, ...) + 
    theme_void() +
    theme(panel.background = element_rect(fill=fill),
          panel.border = element_blank())
}

lower_fn <- function(data, mapping, method="pearson", use="pairwise",...){
  # grab data
  x <- eval_data_col(data, mapping$x)
  y <- eval_data_col(data, mapping$y)
  
  # calculate correlation
  corr <- cor(x, y, method=method, use=use)
  
  # calculate colour based on correlation value
  # Here I have set a correlation of minus one to blue, 
  # zero to white, and one to red 
  # Change this to suit: possibly extend to add as an argument of `my_fn`
  colFn <- colorRampPalette(c(low_color, "white", high_color), interpolate ='spline')
  fill <- colFn(100)[findInterval(corr, seq(0, 1, length=100))]
  ggally_points(data = data, mapping = mapping, ...) + 
    theme_bw() +
    theme(panel.background = element_rect(fill=fill),
          panel.grid.major = element_blank(),
          panel.grid.minor = element_blank())
}

diag_fn <- function(data, mapping, ...){
  ggally_densityDiag(data = data, mapping = mapping, ...) + 
    theme_bw() +
    theme(
          panel.grid.major = element_blank(),
          panel.grid.minor = element_blank())
}
p_pairs <- method_table %>% ggpairs(
  columns = c("relocate2","temp","temp2","tebreak","retroseq","ngs_te_mapper2","teflon","ngs_te_mapper","relocate","popoolationte","te.locate","popoolationte2"),
  lower = list(continuous = wrap(lower_fn, alpha = 0.3, size = 0.1)),
  upper = list(continuous = wrap(upper_fn, color = "black")),
  diag = list(continuous = wrap(diag_fn)),
  labeller = as_labeller(title)
  )
p_leg <- ggplot(data.frame(value = runif(100, min=0, max=1)), aes(x = value, y = value, fill = value)) +
  ylim(c(0,1)) +
  scale_fill_gradient2(low = low_color, mid = "white", high = high_color, midpoint = 0.5, name="Pearson Corr") +
  geom_tile() +
  theme(legend.position = "bottom")

legend <- as_ggplot(get_legend(p_leg))

p <- ggpubr::ggarrange(ggmatrix_gtable(p_pairs), legend, nrow = 2, heights = c(15,1))

ggsave(plot = p,
       paste0(output.location, "/method_correlation_",pub,".pdf"), device = "pdf", width = 15, height = 16, units = "in", dpi = 320)

############################ tRNA table vis ###########################

trnatable <- read.csv(paste(output.location, "/tRNAtable_",pub,".csv", sep = ""), stringsAsFactors = F) %>% 
  melt(id = "X") %>% dplyr::rename( "family" = "X", "method" = "variable" ) %>%
  separate(col = value, into = c("tRNA","all","percentage"), sep = "(/| )") %>%
  mutate(percentage = gsub("\\(|\\)","",percentage,perl = T)) %>%
  mutate(nontRNA = as.numeric(all) - as.numeric(tRNA)) %>%
  # select(-tRNA) %>%
  melt(id = c("family","method"))
trnatable$family <- factor(trnatable$family, levels = c("TY1","TY2","TY3","TY3_1p","TY4","TSU4","TY5"))
trnatable$variable <- factor(trnatable$variable, levels = c("nontRNA","tRNA","all","percentage"))

get_plot_real_trna <- function(x){
  title <- c("ngs_te_mapper","RelocaTE","TEMP","RetroSeq","PoPoolationTE","TE-locate","ngs_te_mapper2","RelocaTE2","TEMP2","TEFLoN","PoPoolationTE2","TEBreak")
  names(title) <- c("ngs_te_mapper","relocate","temp","retroseq","popoolationte","te.locate","ngs_te_mapper2","relocate2","temp2","teflon","popoolationte2","tebreak")
  
  tmp_table <- trnatable %>% filter(method==x) %>% filter(variable=="all")
  tmp_table2 <- trnatable %>% filter(method==x) %>% filter(variable=="percentage")
  trnatable %>% filter(method==x) %>% filter((!variable=="percentage")&(!variable=="all")) %>% ggplot( aes(x=family, y=as.numeric(value), group=variable, fill=variable)) +
    geom_bar(position="stack", stat="identity", width = 0.6) +
    annotate(geom="text", x=tmp_table$family, y=as.numeric(tmp_table$value)+max(as.numeric(tmp_table$value))/40, label=tmp_table2$value, fontface = "plain", size = 3) +
    labs(x = NULL, y = NULL, title = title[x]) +
    guides(fill = guide_legend(title = NULL, nrow = 1, reverse = TRUE)) +
    theme_bw() +
    theme(
      axis.text.x = element_text(angle = 45, vjust = 1, hjust=1),
      axis.text.y = element_text(angle = 90, hjust = 0.5, size = 9),
      plot.title = element_text(hjust = 0.5, size = 14), text = element_text(size = 15)) +
    scale_fill_manual(values = c("nontRNA" = "#fe9929", "tRNA" = "#8c2d04"), labels = c("nontRNA" = "Non-tRNA", "tRNA" = "tRNA")) +
    scale_x_discrete(labels = c("TY1"="Ty1","TY2"="Ty2","TY3"="Ty3","TY3_1p"="Ty3_1p","TY4"="Ty4","TSU4"="Tsu4","TY5"="Ty5"))
}

p_ngs_te_mapper <- get_plot_real_trna("ngs_te_mapper")
p_relocate <- get_plot_real_trna("relocate")
p_temp <- get_plot_real_trna("temp")
p_retroseq <- get_plot_real_trna("retroseq")
p_popoolationte <- get_plot_real_trna("popoolationte")
p_te.locate <- get_plot_real_trna("te.locate")
p_ngs_te_mapper2 <- get_plot_real_trna("ngs_te_mapper2")
p_relocate2 <- get_plot_real_trna("relocate2")
p_temp2 <- get_plot_real_trna("temp2")
p_teflon <- get_plot_real_trna("teflon")
p_popoolationte2 <- get_plot_real_trna("popoolationte2")
p_tebreak <- get_plot_real_trna("tebreak")

p <- ggpubr::ggarrange(p_ngs_te_mapper, p_relocate, p_temp, p_popoolationte, p_retroseq, p_te.locate,
                  p_ngs_te_mapper2, p_relocate2, p_temp2, p_popoolationte2, p_teflon, p_tebreak,
                  ncol = 6, nrow = 2, common.legend = TRUE, legend = "bottom")
p_final <- annotate_figure(p, left = text_grob("Count", rot = 90, size = 15))

ggsave(plot = p_final, paste0(output.location, "/tRNAtable_bar_",pub,".pdf"), device = "pdf", width = 14, height = 8, units = "in", dpi = 320)

############################ real data - tRNA profile ###########################
get_ty_plot <- function(method,TE){
    title <- c("Ty1","Ty2","Ty3","Ty4")
    names(title) <- c("TY1","TY2","TY3","TY4")
    # title2 <- c("ngs_te_mapper","RelocaTE","TEMP","RetroSeq","PoPoolationTE","TE-locate","ngs_te_mapper2","RelocaTE2","TEMP2","TEFLoN","PoPoolationTE2","TEBreak","Coverage")
    # names(title) <- c("ngs_te_mapper","relocate","temp","retroseq","popoolationte","te-locate","ngs_te_mapper2","relocate2","temp2","teflon","popoolationte2","tebreak","coverage")
    
    # path to dist data
    density_dir <- paste0(output.location, "/density_data/")
    dir.create(density_dir, showWarnings = FALSE, recursive = TRUE)

    # tRNA profile plot
    dist_trna <- read.table(paste0(density_dir,"/",TE,"_",method,"_dist.out"), stringsAsFactors = F)
    # den_df <- density(dist_trna$V16, adjust = 0.5)
    #value_1st <- max(den_df$y); id_1st <- den_df$x[which.max(den_df$y == value_1st)]
    #value_2nd <- max(den_df$y < value_1st); id_2nd <- den_df$x[which.max(den_df$y == value_2nd)]
    
    p1 <- ggplot(dist_trna, aes(x = V16,y = ..count..)) +
        xlim(c(-1000,500)) +
        annotate("rect", xmin = -104, xmax = -204, ymin = 0, ymax = Inf, fill = "#82EDED99", alpha = .2) + # first peak at -154
        annotate("rect", xmin = -284, xmax = -374, ymin = 0, ymax = Inf, fill = "#82EDED99", alpha = .2) + # second peak at -324
        annotate("rect", xmin = -450, xmax = -550, ymin = 0, ymax = Inf, fill = "#82EDED99", alpha = .2)+  # third peak at -500
        #geom_tile(data = nuc_data, aes(x = Position,y = -3, fill = Signal), height = 4) + 
        #geom_histogram(binwidth=10, fill = "gray70") +
        geom_density(aes(),color = "brown", adjust = 0.4) +
        theme_bw() +
        scale_fill_gradient(low = "white", high = "#0072B2") +
        theme(axis.text.x = element_blank(), axis.ticks.x = element_blank(),
            axis.text.y = element_text(angle = 90, hjust = 0.5), axis.ticks.y = element_line(),
            plot.title = element_text(hjust = 0.05, size = 14, margin=margin(t=10,b=-20))) +
        labs(x = NULL, y = paste0(title[TE], " Count"), title = NULL) 

    return(p1)
}

# nucleosome occupancy plot
nuc_data <- read.table(paste0(data_dir, "/empirical/nucleosome_data.txt"), col.names = c('Position','Signal'))
p2 <- ggplot(nuc_data, aes(x = Position,y = Signal)) + 
    annotate("rect", xmin = -104, xmax = -204, ymin = 0, ymax = Inf, fill = "#82EDED99", alpha = .2) + # first peak at -154
    annotate("rect", xmin = -284, xmax = -374, ymin = 0, ymax = Inf, fill = "#82EDED99", alpha = .2) + # second peak at -324
    annotate("rect", xmin = -450, xmax = -550, ymin = 0, ymax = Inf, fill = "#82EDED99", alpha = .2) + # third peak at -500
    xlim(c(-1000,500)) +
    geom_line() + 
    theme_bw() + 
    scale_color_grey(start = 0, end = .9) + 
    scale_y_continuous(trans='log10') +
    guides(col=guide_legend(nrow=2)) + 
    theme(axis.text.y = element_text(angle = 90, hjust = 0.5), axis.ticks.y = element_line()) +
    labs(x = "Position relative to tRNA TSS", y = "Nucleosome Occupancy") 

# p2 <- ggplot(nuc_data, aes(x = Position,y = 1, fill = Signal)) + 
#   xlim(c(-1000,500)) +
#   geom_tile() + 
#   theme_bw() + 
#   scale_color_grey(start = 0, end = .9) + 
#   #scale_y_continuous(trans='log10') +
#   guides(col=guide_legend(nrow=2)) + 
#   theme(axis.text.y = element_text(angle = 90, hjust = 0.5), axis.ticks.y = element_line()) +
#   labs(x = "Position relative to tRNA TSS", y = "Nucleosome Occupancy")

# relocate2 plot
p_ty1 <- get_ty_plot("relocate2","TY1")
p_ty2 <- get_ty_plot("relocate2","TY2")
p_ty3 <- get_ty_plot("relocate2","TY3")
p_ty4 <- get_ty_plot("relocate2","TY4")

p <- ggpubr::ggarrange(p_ty1, p_ty2, p_ty3, p_ty4, p2, p2, nrow = 3, ncol = 2, heights = c(2,2,1), legend = "none")
ggsave(plot = p, paste0(output.location, "/nuc_relocate2.pdf"), device = "pdf", width = 8, height = 9, units = "in", dpi = 320)

# ggpubr::ggarrange(p_ty1, p_ty2, p_ty3, p_ty4, p2, nrow = 5, ncol = 1, heights = c(2,2,2,2,1), legend = "none")
# ggsave("~/Github/mcclintock2_paper/docs/manuscript/figures/nuc_relocate2_col1.pdf", device = "pdf", width = 4, height = 12, units = "in", dpi = 320)


## density plots for all methods
supp_density <- function(method){
  title2 <- c("ngs_te_mapper","RelocaTE","TEMP","RetroSeq","PoPoolationTE","TE-locate","ngs_te_mapper2","RelocaTE2","TEMP2","TEFLoN","PoPoolationTE2","TEBreak","Coverage")
  names(title2) <- c("ngs_te_mapper","relocate","temp","retroseq","popoolationte","te-locate","ngs_te_mapper2","relocate2","temp2","teflon","popoolationte2","tebreak","coverage")
  p_ty1 <- get_ty_plot(method,"TY1")
  p_ty2 <- get_ty_plot(method,"TY2")
  p_ty3 <- get_ty_plot(method,"TY3")
  p_ty4 <- get_ty_plot(method,"TY4")
  tmp <- ggpubr::ggarrange(p_ty1, p_ty2, p_ty3, p_ty4, p2, nrow = 5, ncol = 1, heights = c(2,2,2,2,1), legend = "none")
  tmp2 <- annotate_figure(tmp, top = text_grob(title2[method], face = "bold", size = 14))
  return(tmp2)
}

p_ngs <- supp_density("ngs_te_mapper")
p_ngs2 <- supp_density("ngs_te_mapper2")
p_relocate <- supp_density("relocate")
p_relocate2 <- supp_density("relocate2")
p_tebreak <- supp_density("tebreak")
p_temp <- supp_density("temp")
p_temp2 <- supp_density("temp2")
p_retroseq <- supp_density("retroseq")
p_popoolationte <- supp_density("popoolationte")
p_popoolationte2 <- supp_density("popoolationte2")
p_telocate <- supp_density("te-locate")
p_teflon <- supp_density("teflon")

p_mainfig <- ggpubr::ggarrange(p_relocate2, p_temp, p_temp2, p_tebreak, nrow = 1)
ggsave(plot = p_mainfig,
       paste0(output.location, "/main_bestinclass_density.pdf"), device = "pdf", width = 13, height = 12, units = "in", dpi = 320)

p_old <- ggpubr::ggarrange(p_ngs,p_relocate,p_temp,p_popoolationte,p_retroseq,p_telocate, nrow = 1)
ggsave(plot = p_old,
       paste0(output.location, "/supp_old_density.pdf"), device = "pdf", width = 19, height = 12, units = "in", dpi = 320)

p_new <- ggpubr::ggarrange(p_ngs2,p_relocate2,p_temp2,p_popoolationte2,p_teflon,p_tebreak, nrow = 1)
ggsave(plot = p_new,
       paste0(output.location, "/supp_new_density.pdf"), device = "pdf", width = 19, height = 12, units = "in", dpi = 320)

############################ real data - box plot - coverage #############################
cov_table <- read.csv(paste0(output.location, "/coverage_table_for_strain_",pub,".csv"), stringsAsFactors = F) %>%
  mutate_at(vars("TY1_I", "TY2_I", "TY3_I", "TY3_1p_I", "TY4_I", "TSU4_I", "TY5_I"), as.numeric) %>%
  mutate(Count = TY1_I+TY2_I+TY3_I+TY3_1p_I+TY4_I+TSU4_I+TY5_I, Method = "coverage") %>%
  select(-contains("_I")) %>% plyr::rename(c("X" = "Strain"))
setcolors <- c(
  "ngs_te_mapper" = "#E69F0070",
  "ngs_te_mapper2" = "#E69F00",
  "relocate" = "#56B4E970",
  "relocate2" = "#56B4E9",
  "temp" = "#009E7370",
  "temp2" = "#009E73",
  "popoolationte" = "#F0E44270",
  "popoolationte2" = "#F0E442",
  "retroseq" = "#0072B2",
  "te.locate" = "#CCCCCC",
  "teflon" = "#D55E00",
  "tebreak" = "#CC79A7",
  "coverage" = "#DDCC77"
)
p <- cov_table %>% filter(Method == "coverage") %>% ggplot(aes(x = Method, y = Count)) +
  theme_bw() +
  geom_hline(yintercept = 50, lwd = .5, color = "gray40", linetype="dashed") +
  #geom_hline(yintercept = 20, lwd = .5, color = "gray60", linetype="dashed") +
  geom_boxplot(aes(fill = Method), width=0.5) +
  scale_y_continuous(trans='log10', limits = c(1,500),breaks = c(1,2,5,10,20,50,100,200,500)) +
  scale_fill_manual(values = setcolors) +
  labs(x = NULL) +
  theme(plot.title = element_text(hjust = 0.5, size = 14), text = element_text(size = 15),
        axis.text.x = element_blank(), axis.ticks.x = element_blank(),
        strip.background=element_rect(colour="white", fill="white"), strip.text = element_text(angle = 0),
        legend.position = "none")
ggsave(plot = p, paste0(output.location, "/supp_cov_",pub,"_boxplot.pdf"), device = "pdf", width = 3, height = 3.5, units = "in", dpi = 320)
