require(tools)
library(ggplot2)
library(plyr); library(dplyr)
library(tidyr)
library(Rmisc)
library(ggpubr)
library(reshape2)
# library(optparse)
pdf(NULL)

# parse arguments
args = commandArgs(trailingOnly=TRUE)
if (length(args) != 6) {
  stop("Usage: Rscript --vanilla sim_batches_fig.R <paper_dir> <sim1_dir> <sim2_dir> <sim3_dir> <sim4_dir> <out_dir>", call.=FALSE)
} else if (length(args) == 6) {
  paper_dir <- args[1]
  sim1_dir <- args[2]
  sim2_dir <- args[3]
  sim3_dir <- args[4]
  sim4_dir <- args[5]
  out_dir <- args[6]
}

# make output dir
dir.create(out_dir, showWarnings = FALSE, recursive = TRUE)
dir.create(paste0(out_dir, "/figures"), showWarnings = FALSE, recursive = TRUE)
dir.create(paste0(out_dir, "/tables"), showWarnings = FALSE, recursive = TRUE)
# path to reorder py script
reorder_script = paste0(paper_dir, "/simulation_batches/sim_batches_reorder_sum.py")

############################ sim results across coverage and batches, for recall and precision ###########################

sim_precision_recall <- function(p_stat){
  # sim 3
  sim_table_3x <- read.csv(paste0(sim3_dir,"/3/summary/combined_metrics.csv"), stringsAsFactors = F) %>%
    select(contains("precision")|contains("recall")|contains("method")) %>%
    melt(id = "Method") %>%
    mutate(coverage = 3, sim = 3)
  sim_table_6x <- read.csv(paste0(sim3_dir,"/6/summary/combined_metrics.csv"), stringsAsFactors = F) %>%
    select(contains("precision")|contains("recall")|contains("method")) %>%
    melt(id = "Method") %>%
    mutate(coverage = 6, sim = 3)
  sim_table_12x <- read.csv(paste0(sim3_dir,"/12/summary/combined_metrics.csv"), stringsAsFactors = F) %>%
    select(contains("precision")|contains("recall")|contains("method")) %>%
    melt(id = "Method") %>%
    mutate(coverage = 12, sim = 3)
  sim_table_25x <- read.csv(paste0(sim3_dir,"/25/summary/combined_metrics.csv"), stringsAsFactors = F) %>%
    select(contains("precision")|contains("recall")|contains("method")) %>%
    melt(id = "Method") %>%
    mutate(coverage = 25, sim = 3)
  sim_table_50x <- read.csv(paste0(sim3_dir,"/50/summary/combined_metrics.csv"), stringsAsFactors = F) %>%
    select(contains("precision")|contains("recall")|contains("method")) %>%
    melt(id = "Method") %>%
    mutate(coverage = 50, sim = 3)
  sim_table_100x <- read.csv(paste0(sim3_dir,"/100/summary/combined_metrics.csv"), stringsAsFactors = F) %>%
    select(contains("precision")|contains("recall")|contains("method")) %>%
    melt(id = "Method") %>%
    mutate(coverage = 100, sim = 3)
  all_sim3_table_covs <- rbind(sim_table_3x, sim_table_6x,sim_table_12x, sim_table_25x, sim_table_50x, sim_table_100x) %>% 
    separate(col = "variable", into = c("stat","window")) %>%
    mutate(Method = gsub("te-locate","te.locate",Method))
  # sim 4
  sim4_table_3x <- read.csv(paste0(sim4_dir,"/3/summary/combined_metrics.csv"), stringsAsFactors = F) %>%
    select(contains("precision")|contains("recall")|contains("method")) %>%
    melt(id = "Method") %>%
    mutate(coverage = 3, sim = 4)
  sim4_table_6x <- read.csv(paste0(sim4_dir,"/6/summary/combined_metrics.csv"), stringsAsFactors = F) %>%
    select(contains("precision")|contains("recall")|contains("method")) %>%
    melt(id = "Method") %>%
    mutate(coverage = 6, sim = 4)
  sim4_table_12x <- read.csv(paste0(sim4_dir,"/12/summary/combined_metrics.csv"), stringsAsFactors = F) %>%
    select(contains("precision")|contains("recall")|contains("method")) %>%
    melt(id = "Method") %>%
    mutate(coverage = 12, sim = 4)
  sim4_table_25x <- read.csv(paste0(sim4_dir,"/25/summary/combined_metrics.csv"), stringsAsFactors = F) %>%
    select(contains("precision")|contains("recall")|contains("method")) %>%
    melt(id = "Method") %>%
    mutate(coverage = 25, sim = 4)
  sim4_table_50x <- read.csv(paste0(sim4_dir,"/50/summary/combined_metrics.csv"), stringsAsFactors = F) %>%
    select(contains("precision")|contains("recall")|contains("method")) %>%
    melt(id = "Method") %>%
    mutate(coverage = 50, sim = 4)
  sim4_table_100x <- read.csv(paste0(sim4_dir,"/100/summary/combined_metrics.csv"), stringsAsFactors = F) %>%
    select(contains("precision")|contains("recall")|contains("method")) %>%
    melt(id = "Method") %>%
    mutate(coverage = 100, sim = 4)
  all_sim4_table_covs <- rbind(sim4_table_3x, sim4_table_6x,sim4_table_12x, sim4_table_25x, sim4_table_50x, sim4_table_100x) %>% 
    separate(col = "variable", into = c("stat","window")) %>%
    mutate(Method = gsub("te-locate","te.locate",Method))
  all_sim_table_covs <- rbind(all_sim3_table_covs, all_sim4_table_covs)
  get_plot_covs <- function(x){
    # modified title
    title <- c("ngs_te_mapper","RelocaTE","TEMP","RetroSeq","PoPoolationTE","TE-locate","ngs_te_mapper2","RelocaTE2","TEMP2","TEFLoN","PoPoolationTE2","TEBreak")
    names(title) <- c("ngs_te_mapper","relocate","temp","retroseq","popoolationte","te.locate","ngs_te_mapper2","relocate2","temp2","teflon","popoolationte2","tebreak")
    # set colors for lines
    setcolors <- c(
      #"All-nonref" = "#8c2d04",
      #"500" = "#cc4c02",
      #"300" = "#ec7014",
      #"5" = "#fec44f",
      "3" = "#5D3A9B",
      "4" = "#E66100"
    )
    winlabels <- c(
      #"500" = "Within-500",
      #"300" = "Within-300",
      "100" = "Within-100",
      #"5" = "Within-5",
      "0" = "Exact" 
    )
    # set linetypes
    setlinetype <- c(
      "0" = "solid",
      "100" = "dashed"
    )
    simlabels <- c(
      "3" = "tRNA promoter (Sim 3)",
      "4" = "Non-repetitive DNA (Sim 4)"
    )
    # create ggplot
    p <- all_sim_table_covs %>% filter(Method==x & stat==p_stat) %>% filter(window=="100" | window=="0") %>% ggplot( aes(x=coverage, y=value, group=interaction(window,sim), linetype = as.factor(window), color = as.factor(sim))) +
      geom_line(aes()) +
      geom_point(aes(x=coverage,y=value)) +
      scale_color_manual(values = setcolors, labels = simlabels) +
      scale_linetype_manual(values = setlinetype, labels = winlabels) +
      theme_bw() +
      scale_y_continuous(limits = c(0, 1), breaks = c(0.25, 0.5, 0.75,1)) +
      scale_x_continuous(limits = c(0, 100), breaks = c(25, 50, 75,100)) +
      labs(x = NULL, y = NULL, title = title[x]) +
      guides(color = guide_legend(title="Simulation",nrow = 1), linetype = guide_legend(title="Window",nrow = 1)) +
      theme(plot.title = element_text(hjust = 0.5, size = 14), text = element_text(size = 15), legend.position = "bottom")
    
    return(p)
  }
  
  p_ngs_te_mapper <- get_plot_covs("ngs_te_mapper")
  p_relocate <- get_plot_covs("relocate")
  p_temp <- get_plot_covs("temp")
  p_retroseq <- get_plot_covs("retroseq")
  p_popoolationte <- get_plot_covs("popoolationte")
  p_te.locate <- get_plot_covs("te.locate")
  p_ngs_te_mapper2 <- get_plot_covs("ngs_te_mapper2")
  p_relocate2 <- get_plot_covs("relocate2")
  p_temp2 <- get_plot_covs("temp2")
  p_teflon <- get_plot_covs("teflon")
  p_popoolationte2 <- get_plot_covs("popoolationte2")
  p_tebreak <- get_plot_covs("tebreak")
  
  # get legend in a separate file
  legend <- ggpubr::get_legend(p_ngs_te_mapper)
  ggpubr::as_ggplot(legend)
  ggsave(paste0(out_dir,"/figures/legend.pdf"), device = "pdf", width = 24, height = 0.5, units = "cm", dpi = 320)
  
  p_all <- ggpubr::ggarrange(p_ngs_te_mapper, p_relocate, p_temp, p_popoolationte, p_retroseq, p_te.locate,
                    p_ngs_te_mapper2, p_relocate2, p_temp2, p_popoolationte2, p_teflon, p_tebreak,
                    ncol = 6, nrow = 2, common.legend = TRUE, legend = "none")

  p_all2 <- annotate_figure(p_all,
                  left = text_grob(p_stat, rot = 90, size = 25),
                  bottom = text_grob("Fold coverage", size = 15))
  
  ggsave(plot = p_all2, paste0(out_dir,"/figures/sim_",p_stat,"_batches.pdf"), device = "pdf", width = 13, height = 6, units = "in", dpi = 320)
}
sim_precision_recall("Precision")
sim_precision_recall("Recall")

############################ sim results - create sum tables for average numbers ###########################
# generate sim_1 table
system(paste0("python ", reorder_script, " ", sim1_dir, "/100/summary/forward_summary.csv ", sim1_dir, "/100/summary/reverse_summary.csv > ", out_dir, "/tables/sim_1_combined_sum.csv"))
# generate sim_2 table
system(paste0("python ", reorder_script, " ", sim2_dir, "/100/summary/forward_summary.csv ", sim2_dir, "/100/summary/reverse_summary.csv > ", out_dir, "/tables/sim_2_combined_sum.csv"))
# generate sim_3 and sim_4 table
for (cov in c(3,6,12,25,50,100)){
  system(paste0("python ", reorder_script, " ", sim3_dir, "/",cov,"/summary/forward_summary.csv ", sim3_dir, "/",cov,"/summary/reverse_summary.csv > ", out_dir, "/tables/sim_3_",cov,"x_combined_sum.csv"))
  system(paste0("python ", reorder_script, " ", sim4_dir, "/",cov,"/summary/forward_summary.csv ", sim4_dir, "/",cov,"/summary/reverse_summary.csv > ", out_dir, "/tables/sim_4_",cov,"x_combined_sum.csv"))
}

############################ sim results - validation ###########################
nelson_table <- read.csv(paste0(paper_dir,"/simulation_batches/nelson_combined_sum.csv"), stringsAsFactors = F) %>% filter(!X=="Strand") %>% filter(!X=="Reference") %>%
  mutate(ngs_te_mapper = (as.numeric(ngs_te_mapper_x)+as.numeric(ngs_te_mapper_y))/2 ) %>%
  mutate(relocate = (as.numeric(relocate_x)+as.numeric(relocate_y))/2 ) %>%
  mutate(temp = (as.numeric(temp_x)+as.numeric(temp_y))/2 ) %>%
  mutate(retroseq = (as.numeric(retroseq_x)+as.numeric(retroseq_y))/2 ) %>%
  mutate(popoolationte = (as.numeric(popoolationte_x)+as.numeric(popoolationte_y))/2 ) %>%
  mutate(te.locate = (as.numeric(te.locate_x)+as.numeric(te.locate_y))/2 ) %>%
  select(-contains(c("_x","_y"))) %>%
  gather("method", "value", -X) %>%
  mutate(sim = "nelson")
sim1_table <- read.csv(paste0(out_dir, "/tables/sim_1_combined_sum.csv"), stringsAsFactors = F) %>% filter(!X=="Strand") %>% filter(!X=="Reference") %>% filter(!X=="Within-5") %>%
  mutate(ngs_te_mapper = (as.numeric(ngs_te_mapper_x)+as.numeric(ngs_te_mapper_y))/2 ) %>%
  mutate(relocate = (as.numeric(relocate_x)+as.numeric(relocate_y))/2 ) %>%
  mutate(temp = (as.numeric(temp_x)+as.numeric(temp_y))/2 ) %>%
  mutate(retroseq = (as.numeric(retroseq_x)+as.numeric(retroseq_y))/2 ) %>%
  mutate(popoolationte = (as.numeric(popoolationte_x)+as.numeric(popoolationte_y))/2 ) %>%
  mutate(te.locate = (as.numeric(te.locate_x)+as.numeric(te.locate_y))/2 ) %>%
  select(-contains(c("_x","_y"))) %>%
  gather("method", "value", -X) %>%
  mutate(sim = "sim1")
sim2_table <- read.csv(paste0(out_dir, "/tables/sim_2_combined_sum.csv"), stringsAsFactors = F) %>% filter(!X=="Strand") %>% filter(!X=="Reference") %>% filter(!X=="Within-5") %>%
  mutate(ngs_te_mapper = (as.numeric(ngs_te_mapper_x)+as.numeric(ngs_te_mapper_y))/2 ) %>%
  mutate(relocate = (as.numeric(relocate_x)+as.numeric(relocate_y))/2 ) %>%
  mutate(temp = (as.numeric(temp_x)+as.numeric(temp_y))/2 ) %>%
  mutate(retroseq = (as.numeric(retroseq_x)+as.numeric(retroseq_y))/2 ) %>%
  mutate(popoolationte = (as.numeric(popoolationte_x)+as.numeric(popoolationte_y))/2 ) %>%
  mutate(te.locate = (as.numeric(te.locate_x)+as.numeric(te.locate_y))/2 ) %>%
  select(-contains(c("_x","_y"))) %>%
  gather("method", "value", -X) %>%
  mutate(sim = "sim2")
sim3_table <- read.csv(paste0(out_dir, "/tables/sim_3_100x_combined_sum.csv"), stringsAsFactors = F) %>% filter(!X=="Strand") %>% filter(!X=="Reference") %>% filter(!X=="Within-5") %>%
  mutate(ngs_te_mapper = (as.numeric(ngs_te_mapper_x)+as.numeric(ngs_te_mapper_y))/2 ) %>%
  mutate(relocate = (as.numeric(relocate_x)+as.numeric(relocate_y))/2 ) %>%
  mutate(temp = (as.numeric(temp_x)+as.numeric(temp_y))/2 ) %>%
  mutate(retroseq = (as.numeric(retroseq_x)+as.numeric(retroseq_y))/2 ) %>%
  mutate(popoolationte = (as.numeric(popoolationte_x)+as.numeric(popoolationte_y))/2 ) %>%
  mutate(te.locate = (as.numeric(te.locate_x)+as.numeric(te.locate_y))/2 ) %>%
  select(-contains(c("_x","_y"))) %>%
  gather("method", "value", -X) %>%
  mutate(sim = "sim3")

all_validation_tables <- rbind(nelson_table,sim1_table,sim2_table,sim3_table)
all_validation_tables$X <- gsub("Nonreference","All-nonref", all_validation_tables$X)

ylim <- 1.4
sim_table_for_plot <- all_validation_tables %>% mutate(value = case_when(value > ylim ~ ylim, TRUE ~ value))
sim_table_for_plot$X <- factor(sim_table_for_plot$X, levels = c("All-nonref","Within-500","Within-300","Within-100","Within-5","Exact"))
get_plot <- function(x){
  # modified title
  title <- c("ngs_te_mapper","RelocaTE","TEMP","RetroSeq","PoPoolationTE","TE-locate","ngs_te_mapper2","RelocaTE2","TEMP2","TEFLoN","PoPoolationTE2","TEBreak")
  names(title) <- c("ngs_te_mapper","relocate","temp","retroseq","popoolationte","te.locate","ngs_te_mapper2","relocate2","temp2","teflon","popoolationte2","tebreak")

  setcolors <- c(
    "nelson" = "#8c2d04",
    "sim1" = "#cc4c02",
    "sim2" = "#ec7014",
    "sim3" = "#fec44f"
  )
  labels <- c(
    "nelson" = "Nelson et al.",
    "sim1" = "Simulation 1",
    "sim2" = "Simulation 2",
    "sim3" = "Simulation 3, 100x"
  )
  # create ggplot
  p <- sim_table_for_plot %>% filter(method==x) %>% ggplot(aes(y = value, x = X, fill = sim)) +
    geom_bar(position="dodge", stat="identity", width = 0.7) +
    #facet_wrap(~sim) +
    theme_bw() +
    scale_fill_manual(values = setcolors, labels = labels) +
    # ylim(0,ylim[x]) +
    scale_y_continuous(limits = c(0, ylim), breaks = c(0.25, 0.5, 0.75, 1, 1.25)) +
    labs(x = NULL, y = NULL, title = title[x]) +
    theme(axis.text.x = element_text(angle = 45, vjust = 1, hjust=1),
          legend.position="bottom",
          legend.title = element_blank(),
          plot.title = element_text(hjust = 0.5,size = 14),
          text = element_text(size = 12))
  return(p)
  
}

p_ngs_te_mapper <- get_plot("ngs_te_mapper")
p_relocate <- get_plot("relocate")
p_temp <- get_plot("temp")
p_retroseq <- get_plot("retroseq")
p_popoolationte <- get_plot("popoolationte")
p_te.locate <- get_plot("te.locate")

# get legend in a separate file
legend <- ggpubr::get_legend(p_ngs_te_mapper)
ggpubr::as_ggplot(legend)
ggsave(paste0(out_dir,"/figures/legend_valid.pdf"), device = "pdf", width = 24, height = 0.5, units = "cm", dpi = 320)

p_out <- ggpubr::ggarrange(p_ngs_te_mapper, p_relocate, p_temp, p_retroseq, p_popoolationte, p_te.locate,
                          ncol = 3, nrow = 2, common.legend = TRUE, legend = "bottom")
p_out2 <- annotate_figure(p = p_out, left = text_grob("Average number of non-reference predictions", rot = 90, size = 13))
ggsave(plot = p_out2, paste0(out_dir,"/figures/sim_validation.pdf"), device = "pdf", width = 9, height = 7, units = "in", dpi = 320)

############################ copy sim results to final folder ###########################
dir.create(paste0(out_dir, "/figures/sim3"), showWarnings = FALSE, recursive = TRUE)
dir.create(paste0(out_dir, "/figures/sim4"), showWarnings = FALSE, recursive = TRUE)

system(paste0("cp ", sim3_dir, "/r_vis/sim*_50x*.pdf ", out_dir, "/figures/sim3"))
system(paste0("cp ", sim4_dir, "/r_vis/sim*_50x*.pdf ", out_dir, "/figures/sim4"))
system(paste0("cp ", sim3_dir, "/r_vis/sim_*_allcovs.pdf ", out_dir, "/figures/sim3"))
system(paste0("cp ", sim4_dir, "/r_vis/sim_*_allcovs.pdf ", out_dir, "/figures/sim4"))

############################ simulation recall and precision full tables #############################

make_precison_recall_table <- function(sim_num){
  if (sim_num==3) sim_dir <- sim3_dir
  if (sim_num==4) sim_dir <- sim4_dir
  sim_table_3x <- read.csv(paste0(sim_dir, "/3/summary/combined_metrics.csv"), stringsAsFactors = F) %>%
    mutate(Coverage = 3, Simulation = case_when(sim_num==3 ~ "sim3", sim_num==4 ~ "sim4"))
  sim_table_6x <- read.csv(paste0(sim_dir, "/6/summary/combined_metrics.csv"), stringsAsFactors = F) %>%
    mutate(Coverage = 6, Simulation = case_when(sim_num==3 ~ "sim3", sim_num==4 ~ "sim4"))
  sim_table_12x <- read.csv(paste0(sim_dir, "/12/summary/combined_metrics.csv"), stringsAsFactors = F) %>%
    mutate(Coverage = 12, Simulation = case_when(sim_num==3 ~ "sim3", sim_num==4 ~ "sim4"))
  sim_table_25x <- read.csv(paste0(sim_dir, "/25/summary/combined_metrics.csv"), stringsAsFactors = F) %>%
    mutate(Coverage = 25, Simulation = case_when(sim_num==3 ~ "sim3", sim_num==4 ~ "sim4"))
  sim_table_50x <- read.csv(paste0(sim_dir, "/50/summary/combined_metrics.csv"), stringsAsFactors = F) %>%
    mutate(Coverage = 50, Simulation = case_when(sim_num==3 ~ "sim3", sim_num==4 ~ "sim4"))
  sim_table_100x <- read.csv(paste0(sim_dir, "/100/summary/combined_metrics.csv"), stringsAsFactors = F) %>%
    mutate(Coverage = 100, Simulation = case_when(sim_num==3 ~ "sim3", sim_num==4 ~ "sim4"))
  all_sim_table_covs <- rbind(sim_table_3x, sim_table_6x,sim_table_12x, sim_table_25x, sim_table_50x, sim_table_100x)
  all_sim_table_covs <- all_sim_table_covs[,c(28,27,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26)]
  return(all_sim_table_covs)
}
sim3 <- make_precison_recall_table("3")
sim4 <- make_precison_recall_table("4")
all_sim <- rbind(sim3, sim4) %>% mutate_at(vars(matches("Recall|Precision", perl = T)), round, 3)

openxlsx::write.xlsx(all_sim,paste0(out_dir, "/tables/supp_simulation_precision_recall.xlsx"))
write.csv(all_sim,paste0(out_dir, "/tables/supp_simulation_precision_recall.csv"), row.names = F)

############################ simulation average numbers full tables ###########################

make_ave_num_table <- function(sim_num){
  all_sim_table_covs <- data.frame()
  if (sim_num==1) sim_dir <- sim1_dir
  if (sim_num==2) sim_dir <- sim2_dir
  if (sim_num==3) sim_dir <- sim3_dir
  if (sim_num==4) sim_dir <- sim4_dir
  if (sim_num==3 || sim_num==4) {
    for (cov in c(3,6,12,25,50)) {
      sim_table_for <- read.csv(paste0(sim_dir, "/",cov,"/summary/forward_summary.csv"), stringsAsFactors = F, row.names = 1) %>% 
        t() %>% as.data.frame() %>% tibble::rownames_to_column(var = "Method") %>%
        mutate(Coverage = cov, Strand = "+", Simulation = case_when(sim_num==1 ~ "sim1", sim_num==2 ~ "sim2", sim_num==3 ~ "sim3", sim_num==4 ~ "sim4"))
      sim_table_rev <- read.csv(paste0(sim_dir, "/",cov,"/summary/reverse_summary.csv"), stringsAsFactors = F, row.names = 1) %>% 
        t() %>% as.data.frame() %>% tibble::rownames_to_column(var = "Method") %>%
        mutate(Coverage = cov, Strand = "-", Simulation = case_when(sim_num==1 ~ "sim1", sim_num==2 ~ "sim2", sim_num==3 ~ "sim3", sim_num==4 ~ "sim4"))
      all_sim_table_covs <- rbind(all_sim_table_covs,sim_table_for,sim_table_rev)
    }
  }
  sim_table_for <- read.csv(paste0(sim_dir, "/100/summary/forward_summary.csv"), stringsAsFactors = F, row.names = 1) %>% 
    t() %>% as.data.frame() %>% tibble::rownames_to_column(var = "Method") %>%
    mutate(Coverage = 100, Strand = "+", Simulation = case_when(sim_num==1 ~ "sim1", sim_num==2 ~ "sim2", sim_num==3 ~ "sim3", sim_num==4 ~ "sim4"))
  sim_table_rev <- read.csv(paste0(sim_dir, "/100/summary/reverse_summary.csv"), stringsAsFactors = F, row.names = 1) %>% 
    t() %>% as.data.frame() %>% tibble::rownames_to_column(var = "Method") %>%
    mutate(Coverage = 100, Strand = "-", Simulation = case_when(sim_num==1 ~ "sim1", sim_num==2 ~ "sim2", sim_num==3 ~ "sim3", sim_num==4 ~ "sim4"))
  all_sim_table_covs <- rbind(all_sim_table_covs, sim_table_for,sim_table_rev)
  all_sim_table_covs <- all_sim_table_covs[,c(11,9,10,1,2,3,4,5,6,7,8)]
  return(all_sim_table_covs)
}

sim1 <- make_ave_num_table("1")
sim2 <- make_ave_num_table("2")
sim3 <- make_ave_num_table("3")
sim4 <- make_ave_num_table("4")
all_sim <- rbind(sim1, sim2, sim3, sim4) %>% mutate_at(vars(matches("Reference|Nonreference|Exact|Within", perl = T)), round, 3)

openxlsx::write.xlsx(all_sim,paste0(out_dir, "/tables/supp_simulation_average_num.xlsx"))
write.csv(all_sim,paste0(out_dir, "/tables/supp_simulation_average_num.csv"), row.names = F) 

