import pandas as pd
import csv
import argparse
import sys


def main():
    parser = argparse.ArgumentParser(
        description="Combine forward and reverse, reorder columns, print to stdout")
    parser.add_argument(
        "forward_csv", metavar="<forward_csv.csv>", type=str, help="reverse_csv")
    parser.add_argument(
        "reverse_csv", metavar="<reverse_csv.csv>", type=str, help="reverse_csv")

    args = parser.parse_args()
    csv1_file = args.forward_csv
    csv2_file = args.reverse_csv

    pd.options.display.float_format = "{:,.2f}".format
    df_for = pd.read_csv(csv1_file, index_col=0, header=0).astype(float).round(decimals = 2)
    df_for.rename(columns = {'telocate':'te-locate'}, inplace = True)
    df_rev = pd.read_csv(csv2_file, index_col=0, header=0).astype(float).round(decimals = 2)
    df_rev.rename(columns = {'telocate':'te-locate'}, inplace = True)

    if(len(df_for.columns) == 12):
        cols = ["ngs_te_mapper", "relocate", "temp", "retroseq", "popoolationte", "te-locate",
                "ngs_te_mapper2", "relocate2", "temp2", "teflon", "popoolationte2", "tebreak"]
        merged_cols = ["ngs_te_mapper_x", "ngs_te_mapper_y", "relocate_x", "relocate_y", "temp_x", "temp_y", "retroseq_x", "retroseq_y", "popoolationte_x", "popoolationte_y", "te-locate_x", "te-locate_y",
                    "ngs_te_mapper2_x", "ngs_te_mapper2_y", "relocate2_x", "relocate2_y", "temp2_x", "temp2_y", "teflon_x", "teflon_y", "popoolationte2_x", "popoolationte2_y", "tebreak_x", "tebreak_y"]
        strand = pd.DataFrame([["+", "-", "+", "-", "+", "-", "+", "-", "+", "-", "+", "-", 
                                "+", "-", "+", "-", "+", "-", "+", "-", "+", "-", "+", "-"]], columns=merged_cols, index=["Strand"])
    elif(len(df_for.columns) == 6):
        cols = ["ngs_te_mapper", "relocate", "temp",
                "retroseq", "popoolationte", "te-locate"]
        merged_cols = ["ngs_te_mapper_x", "ngs_te_mapper_y", "relocate_x", "relocate_y", "temp_x", "temp_y",
                    "retroseq_x", "retroseq_y", "popoolationte_x", "popoolationte_y", "te-locate_x", "te-locate_y"]
        strand = pd.DataFrame([["+", "-", "+", "-", "+", "-", "+", "-", "+", "-", "+", "-"]], columns=merged_cols, index=["Strand"])
    
    df = df_for.merge(df_rev, how='left', left_index=True,
                        right_index=True)
    df = df.reindex(columns=merged_cols)
    df = strand.append(df)


    df.to_csv(sys.stdout)


if __name__ == "__main__":
    main()
