require(tools)
library(ggplot2)
library(plyr); library(dplyr)
library(tidyr)
library(Rmisc)
library(ggpubr)
library(reshape2)
# library(optparse)
library(UpSetR)
library(grid)
pdf(NULL)

# parse arguments
args = commandArgs(trailingOnly=TRUE)
if (length(args) != 6) {
  stop("Usage: Rscript --vanilla sim_batches_upset.R <paper_dir> <sim1_dir> <sim2_dir> <sim3_dir> <sim4_dir> <out_dir>", call.=FALSE)
} else if (length(args) == 6) {
  paper_dir <- args[1]
  sim1_dir <- args[2]
  sim2_dir <- args[3]
  sim3_dir <- args[4]
  sim4_dir <- args[5]
  out_dir <- args[6]
}

# make output dir
dir.create(out_dir, showWarnings = FALSE, recursive = TRUE)
dir.create(paste0(out_dir, "/figures"), showWarnings = FALSE, recursive = TRUE)
dir.create(paste0(out_dir, "/figures/intersection"), showWarnings = FALSE, recursive = TRUE)

############################ simulation upset plots to show overlap between methods ###########################
# paper_dir = "/home/jc33471/mcclintock2_paper"
# sim3_dir="/scratch/jc33471/mcc_paper/simulation_smk/sim_3"
# sim4_dir="/scratch/jc33471/mcc_paper/simulation_smk/sim_4"
# out_dir="/scratch/jc33471/mcc_paper/simulation_smk/final"
# cov <- 100
# window <- 100
create_upsetplot <- function(cov,window){
    # sim3
    forward <- read.csv(paste0(sim3_dir,"/",cov,"/summary/forward_upsetplot",window,".csv"))
    reverse <- read.csv(paste0(sim3_dir,"/",cov,"/summary/reverse_upsetplot",window,".csv"))
    df_all <- rbind(forward,reverse)
    pdf(file = paste0(out_dir,"/figures/intersection/sim3_upsetplot_",cov,"x_",window,".pdf"), width = 13, height = 13)
    p <- upset(df_all, nsets = 12, number.angles = 30, point.size = 3.5, line.size = 2, 
        mainbar.y.label = "Method Intersections", sets.x.label = "Total TP Per Method", 
        order.by = "freq")
    print(p)
    grid.text(paste0("Simulation 3 Coverage ", cov, " Window ", window),x = 0.65, y=0.95, gp=gpar(fontsize=20))
    dev.off()
    # sim4
    forward <- read.csv(paste0(sim4_dir,"/",cov,"/summary/forward_upsetplot",window,".csv"))
    reverse <- read.csv(paste0(sim4_dir,"/",cov,"/summary/reverse_upsetplot",window,".csv"))
    df_all <- rbind(forward,reverse)
    pdf(file = paste0(out_dir,"/figures/intersection/sim4_upsetplot_",cov,"x_",window,".pdf"), width = 13, height = 13)
    p <- upset(df_all, nsets = 12, number.angles = 30, point.size = 3.5, line.size = 2, 
               mainbar.y.label = "Method Intersections", sets.x.label = "Total TP Per Method", 
               order.by = "freq")
    print(p)
    grid.text(paste0("Simulation 4 Coverage ", cov, "x Window ", window,"bp"),x = 0.65, y=0.95, gp=gpar(fontsize=20))
    dev.off()
}
for (cov in c(3,6,12,25,50,100)){
  for (window in c(0,5,100,300,500)){
    create_upsetplot(cov, window)
  }
}

